@@ -1,7 +1,9 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 1999, 2002, 2003
-// Copyright Dirk Lemstra 2014-2016
+//
+// Copyright @ 2014 ImageMagick Studio LLC, a non-profit organization
+// dedicated to making software imaging solutions freely available.
 //
 // Simple C++ function wrappers for ImageMagick equivalents
 //
@@ -12,13 +14,14 @@
 #include "Magick++/Include.h"
 #include <string>
 
-using namespace std;
-
 #include "Magick++/Functions.h"
 #include "Magick++/Exception.h"
 
+using namespace std;
+
 static bool magick_initialized=false;
 
+// Clone C++ string as allocated C string, de-allocating any existing string
 MagickPPExport void Magick::CloneString(char **destination_,
   const std::string &source_)
 {
@@ -27,28 +30,16 @@ MagickPPExport void Magick::CloneString(char **destination_,
 
 MagickPPExport void Magick::DisableOpenCL(void)
 {
-  GetPPException;
-  MagickCore::InitImageMagickOpenCL(MagickCore::MAGICK_OPENCL_OFF,NULL,NULL,
-    exceptionInfo);
-  ThrowPPException(false);
+  MagickCore::SetOpenCLEnabled(MagickFalse);
 }
 
-MagickPPExport bool Magick::EnableOpenCL(const bool useCache_)
+MagickPPExport bool Magick::EnableOpenCL(void)
 {
   bool
     status;
 
-  GetPPException;
-  if (useCache_)
-    status=MagickCore::InitImageMagickOpenCL(
-      MagickCore::MAGICK_OPENCL_DEVICE_SELECT_AUTO,NULL,NULL,exceptionInfo) ==
-      MagickTrue;
-  else
-    status=MagickCore::InitImageMagickOpenCL(
-      MagickCore::MAGICK_OPENCL_DEVICE_SELECT_AUTO_CLEAR_CACHE,NULL,NULL,
-      exceptionInfo) == MagickTrue;
-  ThrowPPException(false);
-  return(status);
+ status=MagickCore::SetOpenCLEnabled(MagickTrue) != MagickFalse;
+ return(status);
 }
 
 MagickPPExport void Magick::InitializeMagick(const char *path_)
@@ -63,11 +54,23 @@ MagickPPExport void Magick::SetRandomSeed(const unsigned long seed)
   MagickCore::SetRandomSecretKey(seed);
 }
 
+MagickPPExport bool Magick::SetSecurityPolicy(const std::string &policy_)
+{
+  bool
+    status;
+
+  GetPPException;
+  status=MagickCore::SetMagickSecurityPolicy(policy_.c_str(),
+    exceptionInfo) != MagickFalse;
+  ThrowPPException(false);
+  return(status);
+}
+
 MagickPPExport void Magick::TerminateMagick(void)
 {
- if (magick_initialized)
-   {
-     magick_initialized=false;
-     MagickCore::MagickCoreTerminus();
-   }
-}
+  if (magick_initialized)
+    {
+      magick_initialized=false;
+      MagickCore::MagickWandTerminus();
+    }
+}
\ No newline at end of file
