@@ -770,8 +770,12 @@ static boolean ReadAPPProfiles(j_decompress_ptr jpeg_info)
         }
     }
   else
-    status=SetImageProfile(image,"app1",client_info->profiles[marker],
-      exception);
+    {
+      status=SetImageProfile(image,"app1",client_info->profiles[marker],
+        exception);
+      client_info->profiles[marker]=DestroyStringInfo(
+        client_info->profiles[marker]);
+    }
   return(status == MagickFalse ? FALSE : TRUE);
 }
 
@@ -2189,10 +2193,10 @@ static void WriteProfiles(j_compress_ptr jpeg_info,Image *image,
         int
           marker;
 
-        marker=JPEG_APP0+StringToInteger(name+3);
+        marker=APP_MARKER+StringToInteger(name+3);
         for (i=0; i < (ssize_t) length; i+=65533L)
            jpeg_write_marker(jpeg_info,marker,GetStringInfoDatum(profile)+i,
-             MagickMin((unsigned int) length-i,65533L));
+             MagickMin((unsigned int) (length-i),65533));
       }
     else if (LocaleCompare(name,"EXIF") == 0)
       {
@@ -2275,7 +2279,7 @@ static void WriteProfiles(j_compress_ptr jpeg_info,Image *image,
         length=GetStringInfoLength(xmp_profile);
         for (i=0; i < (ssize_t) length; i+=65533L)
           jpeg_write_marker(jpeg_info,APP_MARKER+1,GetStringInfoDatum(
-            xmp_profile)+i,MagickMin((unsigned int) length-i,65533L));
+            xmp_profile)+i,MagickMin((unsigned int) (length-i),65533));
         xmp_profile=DestroyStringInfo(xmp_profile);
       }
     if (image->debug != MagickFalse)
@@ -2348,7 +2352,7 @@ static inline void JPEGSetSample(const struct jpeg_compress_struct *jpeg_info,
   const unsigned int scale,const Quantum pixel,JSAMPLE *q)
 {
   if (jpeg_info->data_precision > 8)
-    (*(unsigned short *) q)=(ScaleQuantumToShort(pixel)/scale);
+    (*(unsigned short *) q)=(unsigned short) (ScaleQuantumToShort(pixel)/scale);
   *q=(JSAMPLE) ScaleQuantumToChar(pixel);
 }
 
