@@ -272,15 +272,15 @@ static Image *ReadTIMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           p=tim_pixels+y*(ssize_t) bytes_per_line;
           for (x=0; x < ((ssize_t) image->columns-1); x+=2)
           {
-            SetPixelIndex(image,(*p) & 0x0f,q);
+            SetPixelIndex(image,(Quantum) ((*p) & 0x0f),q);
             q+=(ptrdiff_t) GetPixelChannels(image);
-            SetPixelIndex(image,(*p >> 4) & 0x0f,q);
+            SetPixelIndex(image,(Quantum) ((*p >> 4) & 0x0f),q);
             p++;
             q+=(ptrdiff_t) GetPixelChannels(image);
           }
           if ((image->columns % 2) != 0)
             {
-              SetPixelIndex(image,(*p >> 4) & 0x0f,q);
+              SetPixelIndex(image,(Quantum) ((*p >> 4) & 0x0f),q);
               p++;
               q+=(ptrdiff_t) GetPixelChannels(image);
             }
